import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;
import java.util.prefs.Preferences;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.FileEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;

/*
 * Cette application appelle un web service avec le contenu de tous les fichiers XML d'un rpertoire.
 * Les rponses seront dposees dans un autre rpertoire.
 * 
 * Seules 3 bibliothques sont ncessaires : commons-logging-x.x.x.jar, httpclient-x.x.jar, et httpcore-x.x.jar, dont la license est
 * "Apache License, Version 2.0" (http://www.apache.org/licenses/LICENSE-2.0) qui est libre et open source.
 * 
 * Yann DEYDIER, mai 2012
 * 
 */

public class SoapMultiCall extends JFrame implements ActionListener, ItemListener {
	private JButton btnLister;
	private JButton btnLancer;
	private JButton btnToutSelectionner;
	private JButton btnQuitter;
	private JButton btnAbout;
	private JTextField txtUrl;
	private JTextField txtRepertoire;
	private JTextField txtRepertoireReponse;
	private JTextField txtSoapAction;
	private JTextField txtTimeout;
	private JCheckBox chkSoapAction;
	private DefaultTableModel tableModel;
	private JTable jTableau;
	private JScrollPane scrollTable;
	private JPanel panel;
	private FrameAbout fenetreAbout;
	// Variables et constantes pour sauvegarde en base de registre (ou autre)
	private Preferences prefs;
	private static final String ID_PREF_URL = "Url";
	private static final String ID_PREF_REPERTOIRE = "Repertoire";
	private static final String ID_PREF_REPERTOIRE_REPONSE = "RepertoireReponses";
	private static final String ID_PREF_SOAP_ACTION = "SoapAction";
	private static final String ID_PREF_SOAP_ACTION_CHK = "SoapActionChk";
	private static final String ID_PREF_TIMEOUT = "Timeout";
	private static final String CODE_OK = "200 (OK)";
	public static final String VERSION = "1.6";

	public SoapMultiCall() {
		super();
		build();
	}

	private void build() {
		setTitle("SoapMultiCall " + VERSION);
		setSize(793, 677);
		setLocationRelativeTo(null); // Centrage de la fentre sur l'cran
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setContentPane(buildContentPane());
	}

	private JPanel buildContentPane() {
		panel = new JPanel();
		panel.setLayout(null);

		// Label titre
		JLabel lblTitre = new JLabel("Soap Multi Call");
		lblTitre.setFont(new Font("Tahoma", Font.BOLD, 17));
		lblTitre.setHorizontalAlignment(SwingConstants.CENTER);
		lblTitre.setBounds(10, 11, 776, 30);
		panel.add(lblTitre);

		// Label prsentation
		JLabel lblPresentation = new JLabel("<html><boby>Cette application appelle un web service (URL) avec le contenu de tous les fichiers XML du r\u00E9pertoire indiqu\u00E9 ci-dessous. Les r\u00E9ponses seront d\u00E9pos\u00E9es dans le 'r\u00E9pertoire des r\u00E9ponses SOAP' (qui sera cr\u00E9\u00E9 automatiquement, dans le 1er r\u00E9pertoire)</body></html>");
		lblPresentation.setFont(new Font("Arial", Font.PLAIN, 12));
		lblPresentation.setBounds(10, 51, 767, 53);
		panel.add(lblPresentation);

		// Rpertoire
		JLabel lblRepertoire = new JLabel("Rpertoire :");
		lblRepertoire.setBounds(10, 133, 87, 14);
		panel.add(lblRepertoire);
		txtRepertoire = new JTextField();
		txtRepertoire.setBounds(209, 130, 444, 20);
		txtRepertoire.setColumns(45);
		panel.add(txtRepertoire);

		// Sous rpertoire des rponses SOAP
		JLabel lblRepertoireReponses = new JLabel("R\u00E9pertoire des r\u00E9ponses SOAP :");
		lblRepertoireReponses.setBounds(10, 161, 193, 14);
		panel.add(lblRepertoireReponses);
		txtRepertoireReponse = new JTextField();
		txtRepertoireReponse.setBounds(209, 158, 444, 20);
		txtRepertoireReponse.setColumns(20);
		panel.add(txtRepertoireReponse);

		// URL
		JLabel lblUrl = new JLabel("URL du WS :");
		lblUrl.setBounds(10, 189, 87, 14);
		panel.add(lblUrl);
		txtUrl = new JTextField();
		txtUrl.setBounds(209, 186, 444, 20);
		txtUrl.setColumns(45);
		panel.add(txtUrl);
		
		// Soap Action
		JLabel lblSoapAction = new JLabel("SoapAction (facultatif) :");
		lblSoapAction.setBounds(10, 215, 141, 14);
		panel.add(lblSoapAction);
		chkSoapAction = new JCheckBox("");
		chkSoapAction.setBounds(206, 212, 27, 20);
		chkSoapAction.setSelected(false);
		chkSoapAction.addItemListener(this);
		panel.add(chkSoapAction);
		txtSoapAction = new JTextField();
		txtSoapAction.setBounds(242, 212, 411, 20);
		txtSoapAction.setColumns(45);
		txtSoapAction.setVisible(false);
		panel.add(txtSoapAction);

		// Timeout
		JLabel lblTimeout = new JLabel("Timeout (secondes) :");
		lblTimeout.setBounds(10, 240, 176, 14);
		panel.add(lblTimeout);
		txtTimeout = new JTextField();
		txtTimeout.setBounds(209, 237, 45, 20);
		txtTimeout.setColumns(4);
		txtTimeout.setVisible(true);
		panel.add(txtTimeout);		
		
		// Bouton Lister
		btnLister = new JButton("1 - Lister Rpertoire");
		btnLister.setBounds(10, 289, 170, 23);
		panel.add(btnLister);
		btnLister.addActionListener(this);

		// Le tableau
		tableModel = new DefaultTableModel();
		tableModel.addColumn("Nom fichier entre SOAP");
		tableModel.addColumn("Retour SOAP");
		tableModel.addColumn("Code Retour");
		tableModel.addColumn("Erreur");
		jTableau = new JTable(tableModel) {
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		jTableau.getColumnModel().getColumn(0).setPreferredWidth(300);
		jTableau.getColumnModel().getColumn(1).setPreferredWidth(300);
		jTableau.getColumnModel().getColumn(2).setPreferredWidth(100);
		jTableau.getColumnModel().getColumn(3).setPreferredWidth(200);
		jTableau.getTableHeader().setReorderingAllowed(false);
		for (int i=0;i<4;i++) {
			jTableau.getColumnModel().getColumn(i).setCellRenderer(new CellRenderer());
		}
		jTableau.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				// Gestion du double click sur le tableau
				if (e.getClickCount() == 2) {
					JTable target = (JTable) e.getSource();
					int row = target.getSelectedRow();
					int column = target.getSelectedColumn();
					if (column == 0) {
						String strNomFichier = jTableau.getValueAt(row, column).toString();
						String strRepertoire = txtRepertoire.getText().trim();
						strRepertoire = repertoireAvecSlash(strRepertoire);
						String strCheminFichierComplet = strRepertoire + strNomFichier;
						ouvrirFichier(strCheminFichierComplet);
					}
					if (column == 1) {
						String strNomFichier = jTableau.getValueAt(row, column).toString();
						String strRepertoire = txtRepertoire.getText().trim();
						strRepertoire = repertoireAvecSlash(strRepertoire);
						String strRepertoireReponse = txtRepertoireReponse.getText().trim();
						strRepertoireReponse = strRepertoire + strRepertoireReponse;
						strRepertoireReponse = repertoireAvecSlash(strRepertoireReponse);
						String strCheminFichierComplet = strRepertoireReponse + strNomFichier;
						ouvrirFichier(strCheminFichierComplet);
					}
					if ((column == 2) || (column == 3)) {
						String strContenuCellule = jTableau.getValueAt(row, column).toString();
						FramePopup fenetre = new FramePopup(strContenuCellule);
						fenetre.setVisible(true);
					}
				}
			}
		});
		scrollTable = new JScrollPane(jTableau);
		scrollTable.setBounds(10, 323, 764, 223);
		panel.add(scrollTable);

		// Label double cliquez
		JLabel labelDouble = new JLabel("Double cliquez sur un nom de fichier pour le visualiser.");
		labelDouble.setHorizontalAlignment(SwingConstants.RIGHT);
		labelDouble.setBounds(432, 557, 345, 14);
		panel.add(labelDouble);

		// Bouton tout slectionner
		btnToutSelectionner = new JButton("Tout slectionner");
		btnToutSelectionner.setBounds(10, 557, 141, 23);
		btnToutSelectionner.setEnabled(false);
		btnToutSelectionner.addActionListener(this);
		panel.add(btnToutSelectionner);

		// Bouton lancer appel SOAP
		btnLancer = new JButton("2 - Lancer les appels SOAP sur les lignes slectionnes");
		btnLancer.setBounds(10, 615, 390, 23);
		btnLancer.setEnabled(false);
		panel.add(btnLancer);
		btnLancer.addActionListener(this);
		
		// Bouton About
		btnAbout = new JButton("About");
		btnAbout.setBounds(585, 615, 68, 23);
		btnAbout.addActionListener(this);
		panel.add(btnAbout);

		// Bouton Quitter
		btnQuitter = new JButton("Quitter");
		btnQuitter.setBounds(659, 615, 118, 23);
		panel.add(btnQuitter);
		btnQuitter.addActionListener(this);

		chargerPreferences();
		return panel;
	}

	@Override
	public void actionPerformed(ActionEvent arg) {
		Object source = arg.getSource();

		// Lancer les appels SOAP
		if (source == btnLancer) {
			actionLancer();
		}

		// Lister le rpertoire
		if (source == btnLister) {
			actionLister();
		}

		// Tout slectionner
		if (source == btnToutSelectionner) {
			jTableau.getSelectionModel().addSelectionInterval(0, tableModel.getRowCount() - 1);
		}
		
		// About
		if (source == btnAbout) {
			if (fenetreAbout==null) fenetreAbout = new FrameAbout();
			fenetreAbout.setVisible(true);
		}

		// Quitter
		if (source == btnQuitter) {
			System.exit(0);
		}
	}

	
	// ************************************************************************
	// ************************ Action Bouton LANCER **************************
	// ************************************************************************
	private void actionLancer() {
		sauverPreferences();
	
		String strUrl = txtUrl.getText().trim();
		String strNomFichier = "";

		// Lignes slectionnes
		int[] ligneSelectionnees = jTableau.getSelectedRows();
		if (ligneSelectionnees.length == 0) {
			JOptionPane.showMessageDialog(null, "Veuillez slectionner au moins une ligne.", "Information", JOptionPane.INFORMATION_MESSAGE);
			return;
		}
	
		// Vidage de la table
		for (int row = 0; row < ligneSelectionnees.length; row++) {
			jTableau.setValueAt("", ligneSelectionnees[row], 1);
			jTableau.setValueAt("", ligneSelectionnees[row], 2);
			jTableau.setValueAt("", ligneSelectionnees[row], 3);
		}
		rafraichirTableau(); // repaint
	
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		// Parcours des lignes slectionnes
		for (int row = 0; row < ligneSelectionnees.length; row++) {
			strNomFichier = jTableau.getValueAt(ligneSelectionnees[row], 0).toString();
			String strRepertoire = txtRepertoire.getText().trim();
			strRepertoire = repertoireAvecSlash(strRepertoire);
			String strCheminFichierComplet = strRepertoire + strNomFichier;
			String strSoapAction = "";
			if (chkSoapAction.isSelected()) strSoapAction = txtSoapAction.getText().trim();
			// Appel HTTP
			RetourHttp retourHttp = appelHttp(strCheminFichierComplet, strUrl, strSoapAction);
			// Traitement du retour
			String strCodeRetour = retourHttp.getCodeRetour();
			String strMsgRetour = retourHttp.getMsgRetour();
			if (strCodeRetour.equals("200")) {
				strCodeRetour = CODE_OK;
				strMsgRetour = "";
			}
			// Remplissage du tableau
			jTableau.setValueAt(strCodeRetour, ligneSelectionnees[row], 2);
			jTableau.setValueAt(strMsgRetour, ligneSelectionnees[row], 3);
			rafraichirTableau();
			// Ecriture du fichier contenant le rsultat
			String strNomFichierResultat="_REPONSE_SOAP_"+strNomFichier;
			String strRepertoireReponse = txtRepertoireReponse.getText().trim();
			strRepertoireReponse = strRepertoire + strRepertoireReponse; 
			strRepertoireReponse = repertoireAvecSlash(strRepertoireReponse);
			File repReponse = new File(strRepertoireReponse);
			if (!repReponse.exists()) repReponse.mkdir();
			if ((retourHttp.getContenu()!=null) && (!retourHttp.getContenu().trim().equals(""))) {
				ecrireFichier(strRepertoireReponse + strNomFichierResultat, retourHttp.getContenu());
				jTableau.setValueAt(strNomFichierResultat, ligneSelectionnees[row], 1);
			}
		}
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}

	// ************************************************************************
	// ************************ Action Bouton LISTER **************************
	// ************************************************************************
	private void actionLister() {
		sauverPreferences();
		// Vidage de la table
		int n = tableModel.getRowCount();
		for (int i = 0; i < n; i++) tableModel.removeRow(0);
		// Remplissage de la table
		File[] listeFichiers;
		String strRepertoire = txtRepertoire.getText();
		File repertoire = new File(strRepertoire);
		listeFichiers = repertoire.listFiles();
		if (listeFichiers == null) {
			btnLancer.setEnabled(false);
			btnToutSelectionner.setEnabled(false);
			JOptionPane.showMessageDialog(null, "Rpertoire introuvable !", "Erreur", JOptionPane.ERROR_MESSAGE);
			return;
		}
		for (int i = 0; i < listeFichiers.length; i++) {
			File aFile = listeFichiers[i];
			if (aFile.isFile()) {
				Vector vLigne = new Vector();
				vLigne.add(aFile.getName());
				vLigne.add("");
				vLigne.add("");
				vLigne.add("");
				tableModel.addRow(vLigne);
			}
		}
		if (listeFichiers.length > 0) {
			btnLancer.setEnabled(true);
			btnToutSelectionner.setEnabled(true);
		} else {
			btnLancer.setEnabled(false);
			btnToutSelectionner.setEnabled(false);
		}
	}
	
	// Sauvegarde des donnes en base de registre (ou autre, selon l'OS utilis)
	private void sauverPreferences() {
		prefs = Preferences.userRoot().node(this.getClass().getName());
		prefs.put(ID_PREF_URL, txtUrl.getText());
		prefs.put(ID_PREF_REPERTOIRE, txtRepertoire.getText());
		prefs.put(ID_PREF_REPERTOIRE_REPONSE, txtRepertoireReponse.getText());
		prefs.put(ID_PREF_SOAP_ACTION, txtSoapAction.getText());
		prefs.put(ID_PREF_SOAP_ACTION_CHK, (chkSoapAction.isSelected()?"OUI":"NON"));
		prefs.put(ID_PREF_TIMEOUT, txtTimeout.getText());
	}

	private void chargerPreferences() {
		prefs = Preferences.userRoot().node(this.getClass().getName());
		txtUrl.setText(prefs.get(ID_PREF_URL, ""));
		txtRepertoire.setText(prefs.get(ID_PREF_REPERTOIRE, ""));
		txtRepertoireReponse.setText(prefs.get(ID_PREF_REPERTOIRE_REPONSE, ""));
		if (txtRepertoireReponse.getText().equals("")) txtRepertoireReponse.setText("ReponsesSoap"); 
		boolean bSAChecked = (prefs.get(ID_PREF_SOAP_ACTION_CHK, "NON").equals("OUI"));
		chkSoapAction.setSelected(bSAChecked);
		txtSoapAction.setText(prefs.get(ID_PREF_SOAP_ACTION, ""));
		activerDesactiverTxtSoapAction();
		txtTimeout.setText(prefs.get(ID_PREF_TIMEOUT, "30"));
	}

	// Appel HTTP
	private RetourHttp appelHttp(String nomFichierComplet, String strUrl, String strSoapAction) {
		RetourHttp retourHttp = new RetourHttp();

		File file = new File(nomFichierComplet);
		FileEntity entity = new FileEntity(file, ContentType.create("text/xml", "UTF-8"));
		HttpPost httppost = new HttpPost(strUrl);
		httppost.setEntity(entity);
		httppost.addHeader("SOAPAction", strSoapAction);

		// PROXY ventuel !
		// HttpHost proxy = new HttpHost("localhost", 8888);
		// httpclient.getParams().setParameter(ConnRoutePNames.DEFAULT_PROXY, proxy);

		try {
			HttpClient httpclient = new DefaultHttpClient();
			Integer timeout = new Integer(txtTimeout.getText()) * 1000;
			httpclient.getParams().setParameter("http.socket.timeout", timeout); // time out
			HttpResponse response = httpclient.execute(httppost);
			retourHttp.setMsgRetour(response.getStatusLine().toString());
			retourHttp.setCodeRetour(Integer.toString(response.getStatusLine().getStatusCode()));
			HttpEntity entityReponse = response.getEntity();
			if (entityReponse != null) {
				retourHttp.setContenu(EntityUtils.toString(entityReponse));
			}
		} catch (Exception e) {
			retourHttp.setMsgRetour(e.toString());
			retourHttp.setCodeRetour("KO");
		}
		return retourHttp;
	}

	public class CellRenderer extends DefaultTableCellRenderer {
		@Override
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
			super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
			if ((column==2) || (column==3)) {
				String codeRetour=jTableau.getValueAt(row, 2).toString();
				if ((!codeRetour.equals("")) && (!codeRetour.equals(CODE_OK))) {
					setForeground(new Color(230,0,0,255));
				} else {
					setForeground(new Color(0,130,0,255));
				}
			}
			if (column==2) {
				setHorizontalAlignment( SwingConstants.CENTER );
			}
			return this;
		}
	}

	// Provoque un "repaint" du tableau
	private void rafraichirTableau() {
		jTableau.paintImmediately(jTableau.getVisibleRect());
	}

	public static void ouvrirFichier(String filename) {
		File file = new File(filename);
		if (!file.exists() && file.length() < 0) {
			System.out.println("Specified file does not exist!");
			return;
		}
		Desktop desktop = null;
		if (Desktop.isDesktopSupported()) {
			desktop = Desktop.getDesktop();
		}
		try {
			desktop.open(file);
		} catch (IOException ex) {
		}
	}

	// Ajoute un slash (/)  la fin d'un nom de rpertoire, s'il n'y s'y trouve pas
	public static String repertoireAvecSlash(String strRepertoire) {
		String strRetour = strRepertoire;
		String dernierCaractere = strRepertoire.substring(strRepertoire.length() - 1, strRepertoire.length());
		if ((!dernierCaractere.equals("/")) && (!dernierCaractere.equals("\\"))) {
			strRetour = strRetour + "/";
		}
		return strRetour;
	}
	
	// Ecrit "contenu" dans le fichier "cheminFichier" 
	private static void ecrireFichier(String cheminFichier, String contenu) {
	    try {
	    	PrintWriter ecrivain;
	    	ecrivain =  new PrintWriter(new BufferedWriter(new FileWriter(cheminFichier)));
		    ecrivain.print(contenu);
		    ecrivain.close();		
	    } catch (Exception e) {
	    }
	}
	
	 // Listens to the check box
    public void itemStateChanged(ItemEvent e) {
        Object source = e.getItemSelectable();
        if (source == chkSoapAction) {
        	activerDesactiverTxtSoapAction();
        }
    }
    
    private void activerDesactiverTxtSoapAction() {
    	if (chkSoapAction.isSelected()) {
    		txtSoapAction.setVisible(true);
    	} else {
    		txtSoapAction.setVisible(false);
    	}
    }
}